<?php
/**
 * ## Kurzanleitung (Langform in beiliegender Datei) ##
 *
 * 1. Variablen _VERNETZUNGSCACHE, _ANBIETERKENNUNG und _KONTAKTEMAIL im
 *    veraenderbaren Bereich definieren. Optional sind die restlichen
 *    Variablen.
 * 2. Diese Datei in das unter _VERNETZUNGSCACHE definierte, manuell angelegte
 *    Verzeichnis legen (/dejure.org/ ist ein Vorschlag, moeglich und - je nach
 *    System - naheliegender waeren evtl. /var/cache/dejure.org/ oder
 *    /home/benutzer/dejure.org/)
 * 3. Diese Datei mit include_once _vor_ dem zu vernetzenden Textbereich in der
 *    einschlaegigen Vorlagendatei einbinden. Z. B.:
 *    < ?php include_once('/dejure.org/vernetzungsfunction.inc.php'); ? >
 *    Leerzeichen vor und nach den spitzen Klammern nicht zu entfernen
 *    vergessen!)
 * 4. _Nach_ dem zu vernetzenden Textbereich einfuegen:
 *    < ?php ob_end_flush(); ? >
 *    Ohne die Leerzeichen vor und nach den spitzen Klammern!
 *
 * HINWEIS zum eingebauten Cache der Vernetzung:
 *    Der Datei-Cache ist zwar simpel und funktioniert fast ueberall, doch wird er
 *    bei vielen Dateien langsam.
 *    Wenn Sie viele Texte mit vernetzbaren Inhalten haben (ca. ab 500), sollten
 *    Sie einen Datenbank-Cache in Erwaegung ziehen.
 *    Da dies je nach Server/Webspace andere Anforderungen stellt, ist es nicht
 *    standardmaessig eingebaut. Im Plugin fuer Wordpress finden Sie eine Loesung
 *    per MySQL:
 *    https://dejure.org/vernetzung/dejure.org-vernetzung-wordpress.zip
 *    Auf Anfrage helfen wir aber gern.
 */


/* ++++++++ Start veraenderbarer Bereich ++++++++++++++++++++++++++++++++++++ */

// [Standard: /dejure.org/] Relativer oder absoluter Name eines (les- und
// beschreibbaren! - Attribute: etwa 777) Verzeichnisses, mit abschliessendem
// "/" (bei leerer Variablen laeuft die Funktion ohne Cache)
#define('_VERNETZUNGSCACHE',	'/dejure.org/');
define('_VERNETZUNGSCACHE',	'./tmp/');

// [Standard: X] Hier koennen Sie eine eigene, feste Bezeichnung fuer Ihren Dienst
// zur Kommunikation mit dejure.org festlegen (Buchstaben und Zahlen erlaubt,
// mindestens 5 Stellen). Wird im Rahmen der Serverkommunikation jeweils an
// dejure.org uebertragen und dient im uebrigen ausschliesslich der Kommunikation
// bei technischen Problemen.
define('_ANBIETERKENNUNG',	$_SERVER['HTTP_HOST']); // Domain des Webangebots
define('_KONTAKTEMAIL',		'webmaster@'.$_SERVER['HTTP_HOST']);

// [Standard: weit] schmal: nur die Nummern der Vorschriften verlinken.
// weit: moeglichst lange Verlinkung
define('_LINKSTIL',			'weit');

// [Standard: leer] Zum Beispiel fuer oeffnen in neuem Fenster: "_blank"
define('_TARGET',			'');

// [Standard: leer] CSS Klasse. Falls Sie die Links zu dejure.org anders
// gestalten moechten
define('_CLASS',			'');

// [Standard: 1] Zu http://buzer.de verlinken fuer Gesetze, die bei dejure.org
// nicht vorhanden sind. 0 zum Deaktivieren.
define('_BUZER',			1);

// [Standard: 2] In Tagen. Wie lange die Texte im Cache vorgehalten werden soll,
// bevor sie erneut vernetzt werden. 4 Tage reichen in der Regel.
define('_CACHE_DAUER',		2);

// [Standard: 3] In Sekunden. Wie lange versucht wird eine Verbindung zu
// dejure.org aufzubauen. Standwardwert ist normalerweise ausreichend.
define('_ZEITLIMIT',		3);

/* ++++++++ Ende veraenderbarer Bereich +++++++++++++++++++++++++++++++++++++ */


define('DJO_VERSION', '2.22');

/**
 * djo_ausgabepuffer
 *
 * Hauptfunktion. Initiiert Vernetzung.
 * Prueft vorher auf zu vernetzende Inhalte.
 * Bricht ab wenn nichts zu tun ist.
 * @return String mit dem vernetzten/unvernetzten Text.
 */
function djo_ausgabepuffer($ausgabetext = '') {
	// zu vernetzende Inhalte im Text? Ansonsten Originaltext zurueck
	if (!preg_match("/§|&sect;|Art\.|\/[0-9][0-9](?![0-9\/])| [0-9][0-9]?[\/\.][0-9][0-9](?![0-9\.])|[0-9][0-9], /", $ausgabetext)) {
		return $ausgabetext;
	}
	$arr_parameter = array(
		'Anbieterkennung'			=> _ANBIETERKENNUNG.'__'._KONTAKTEMAIL,
		'format'					=> _LINKSTIL,
		'target'					=> _TARGET,
		'class'						=> _CLASS,
		'buzer'						=> _BUZER,
		'zeitlimit_in_sekunden'		=> _ZEITLIMIT,
		'version'					=> 'php-'.DJO_VERSION,
		'Schema'					=> 'https',
	);

	// Abfragen des Cache, ob der Text schon gespeichert ist
	$rueckgabe = vernetzen_ueber_cache($ausgabetext);
	if (empty($rueckgabe)) {
		// Kein Cache, Anfrage an dejure.org senden
		$rueckgabe = vernetzen_ueber_dejure_org($ausgabetext, $arr_parameter);
	}
	// Einmal zwischen 0 und 6 Uhr alle Dateien im Cacheverzeichnis loeschen,
	// die aelter als X Tage sind. Dies loest regelmaessig eine aktualisierende
	// Neuvernetzung aus
	if (date('G') < 6) {
		cache_dateien_loeschen();
	}
	return $rueckgabe;
}

/**
 * vernetzen_ueber_cache
 *
 * Holt eventuell vorhandenen vernetzten Text aus dem Cache.
 * @return String mit vernetzten Text oder Bool false
 */
function vernetzen_ueber_cache($ausgangstext) {
	// Cache-Verzeichnis und Cache-Dauer definiert?
	if (defined('_VERNETZUNGSCACHE') && defined('_CACHE_DAUER')) {
		$vernetzungscache = _VERNETZUNGSCACHE;
		$cache_dauer = _CACHE_DAUER;
	}
	if (empty($vernetzungscache) || !is_dir($vernetzungscache) || !is_numeric($cache_dauer)) {
		return false;
	}
	$ausgangstext = trim($ausgangstext);
	// Cache Vorhaltedauer in Sekunden wandeln
	$cache_dauer = $cache_dauer*86400;
	$schluessel = strlen($ausgangstext).md5($ausgangstext);
	if (file_exists($vernetzungscache.$schluessel)) {
		// Datei aelter als Cache-Dauer? Neu vernetzen
		if (filemtime($vernetzungscache.$schluessel) < time()-$cache_dauer) {
			return false;
		}
		// Cache-Datei auslesen und zurueck geben
		return file_get_contents($vernetzungscache.$schluessel);
	}
	// Kein Cache vorhanden. Neu vernetzen
	return false;
}

/**
 * ablegen_in_cache
 *
 * Legt vernetzten Text im Cache ab.
 * @return none
 */
function ablegen_in_cache($ausgangstext, $rueckgabe) {
	if (defined('_VERNETZUNGSCACHE')) {
		$vernetzungscache = _VERNETZUNGSCACHE;
	}
	if (!empty($vernetzungscache) && is_dir($vernetzungscache)) {
		$schluessel = strlen($ausgangstext).md5($ausgangstext);
		$d = fopen($vernetzungscache.$schluessel, "w");
		if ($d !== false) {
			fwrite($d, $rueckgabe);
			fclose($d);
		}
	}
}

/**
 * cache_dateien_loeschen
 *
 * Entfernt veraltete Cache-Dateien
 * @return Array der geloeschten Dateien fuer Debug-Zwecke
 */
function cache_dateien_loeschen() {
	if (defined('_VERNETZUNGSCACHE') && defined('_CACHE_DAUER')) {
		$vernetzungscache = _VERNETZUNGSCACHE;
		$cache_dauer = _CACHE_DAUER;
	}
	if (empty($vernetzungscache) || !is_dir($vernetzungscache) || !is_numeric($cache_dauer)) {
		return false;
	}
	// Cache Vorhaltedauer in Sekunden wandeln
	$cache_dauer = $cache_dauer*86400;

	// Zeitpunkt des letzten Loeschens abfragen
	if (!file_exists($vernetzungscache.'cache_status')) {
		cache_status_datei();
		return;
	}
	$cache_status = file_get_contents($vernetzungscache.'cache_status');

	if (time() - $cache_status > 86400) { // 24*60*60 -- Wenn letztes Loeschen 24h zurueck liegt
		$dateien = scandir($vernetzungscache);
		$geloescht = array();
		if (!empty($dateien[0])) {
			foreach ($dateien as $datei) {
				if (!in_array($datei, array(".", "..", "vernetzungsfunction.inc.php"))) {
					$file_time = filemtime($vernetzungscache.$datei);
					// Datei aelter als Vorhaltedauer Stunden, dann loeschen
					if ($file_time < (time() - $cache_dauer)) {
						unlink($vernetzungscache.$datei);
						$geloescht[] = $vernetzungscache.$datei;
					}
				}
			}
		}
		// Status-Datei neu erzeugen
		cache_status_datei($verzeichnis);
		return $geloescht;
	}
}

/**
 * cache_status_datei
 *
 * Legt eine Datei mit dem letzten Zeitpunkt der Cache-Saeuberung an
 * @return none
 */
function cache_status_datei() {
	$cache_zeitstempel = mktime(0, 0, 0, date('d'), date('m'), date('Y'));
	$file_handle = fopen(_VERNETZUNGSCACHE.'cache_status', 'w');
	fputs($file_handle, $cache_zeitstempel);
	fclose($file_handle);
}

/**
 * integritaetskontrolle_und_cache
 *
 * Prueft, ob der von der Vernetzung zurueck gegebene Text mit entfernten
 * dejure.org Links dem Originaltext gleicht. Unterscheiden sich wenn es
 * einen Fehler bei der Vernetzung gab.
 * @return String mit dem Originaltext
 */
function integritaetskontrolle_und_cache($ausgangstext, $neuertext) {
	// pruefen, ob zurueckgegebener, vernetzter Text bis auf die hinzugefuegten Links dem Original gleicht
	$neuertext		= trim($neuertext);
	$ausgangstext	= trim($ausgangstext);
	if (preg_replace("/<a href=\"https?:\/\/dejure.org\/[^>]*>([^<]*)<\/a>/i", "\\1", $ausgangstext) == preg_replace("/<a href=\"https?:\/\/dejure.org\/[^>]*>([^<]*)<\/a>/i", "\\1", $neuertext)) {
		// Alles in Ordnung. Vernetzten Text im Cache speichern und zurueck geben
		ablegen_in_cache($ausgangstext, $neuertext);
		return $neuertext;
	} else {
		// Texte unterschiedlich. Originaltext zurueck geben
		return $ausgangstext;
	}
}

/**
 * vernetzen_ueber_dejure_org
 *
 * Sendet den Originaltext an dejure.org und erhaelt den vernetzten Text.
 * Erste Pruefung, ob Vernetzung fehlerfrei ablief.
 * @return String Vernetzter Text bei Erfolg. Ansonsten Originaltext.
 */
function vernetzen_ueber_dejure_org($ausgangstext, $parameter = array()) {
	// Moegliche Parameter: Anbieterkennung / Dokumentkennung / target / class / linkstil / zeitlimit_in_sekunden
	// Hinweis: Bei Aenderung dieser Einstellungen muss der Cache manuell geloescht werden
	$ausgangstext = trim($ausgangstext);
	$uebergabe = 'Originaltext='.urlencode($ausgangstext);
	foreach ($parameter as $option => $wert) {
		if ($option == 'zeitlimit_in_sekunden') {
			$zeitlimit_in_sekunden = $wert;
		} else {
			$uebergabe.= '&'.urlencode($option).'='.urlencode($wert);
		}
	}
	if (empty($zeitlimit_in_sekunden) || !is_numeric($zeitlimit_in_sekunden)) {
		$zeitlimit_in_sekunden = 3;
	}
	$header = 'POST /dienste/vernetzung/vernetzen HTTP/1.0'."\r\n";
	$header .= 'User-Agent: '._ANBIETERKENNUNG.' (PHP-Vernetzung '.DJO_VERSION.')'."\r\n";
	$header .= 'Content-type: application/x-www-form-urlencoded'."\r\n";
	$header .= 'Content-length: '.strlen($uebergabe)."\r\n";
	$header .= 'Host: rechtsnetz.dejure.org'."\r\n";
	$header .= 'Connection: close'."\r\n";
	$header .= "\r\n";

	if (extension_loaded('openssl')) {
		$fp = fsockopen('tls://rechtsnetz.dejure.org', 443, $errno, $errstr, $zeitlimit_in_sekunden);
	}
	if (!$fp) {
		$fp = fsockopen('rechtsnetz.dejure.org', 80, $errno, $errstr, $zeitlimit_in_sekunden);
	}

	if (!$fp) { // Verbindung gescheitert. Originaltext zurueck geben
		return $ausgangstext;
	} else {
		stream_set_timeout($fp, $zeitlimit_in_sekunden, 0); // Verbindung nach $zeitlimit_in_sekunden Sekunden abbrechen
		stream_set_blocking($fp, true);
		fputs($fp, $header.$uebergabe);
		$timeOutSock = false;
		$eofSock = false;
		$rueckgabe='';
		while (!$eofSock && !$timeOutSock) {
			$rueckgabe.= fgets($fp, 1024); //
			$stSock = stream_get_meta_data($fp);
			$eofSock = $stSock['eof'];
			$timeOutSock = $stSock['timed_out'];
		}
		fclose($fp);
		if (!preg_match("/^(.*?)\r?\n\r?\n\r?\n?(.*)/s",$rueckgabe, $rueckgabeARR)) {
			return $ausgangstext; // Zeitueberschreitung oder Verbindungsproblem
		} else if (strpos($rueckgabeARR[1],'200 OK') === false) {
			return $ausgangstext; // sonstiges Serverproblem
		} else {
			$rueckgabe = $rueckgabeARR[2];
			// Vernetzter Text kleiner als Ausgangstext. Eventuell Fehler bei Uebertragung
			if (strlen($rueckgabe) < strlen($ausgangstext)) {
				return $ausgangstext;
			}
			// Vernetzung erfolgreich. Rueckgabe auf Fehler pruefen
			return integritaetskontrolle_und_cache($ausgangstext, $rueckgabe);
		}
	}
}
ob_start('djo_ausgabepuffer');
?>
